@@ -20,13 +20,13 @@
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "libdb/db_storage.h"
+#include "db_storage.h"
 
 /* check_mandirs.c */
 extern void test_manfile (const char *file, const char *path);
 extern void update_db_time (void);
 extern void reset_db_time (void);
-extern short create_db (const char *manpath);
-extern short update_db (const char *manpath);
+extern int create_db (const char *manpath);
+extern int update_db (const char *manpath);
 extern void purge_pointers (const char *name);
-extern short purge_missing (const char *manpath, const char *catpath);
+extern int purge_missing (const char *manpath, const char *catpath);
